from __future__ import unicode_literals

import logging
import time
from datetime import datetime
from dateutil import parser as date_parser
import inspect
import six

import pymisp
import requests
from .otx import OTXv2

try:
  basestring
except NameError:
  basestring = str

__version__ = "1.4.2"

# Try to disable verify SSL warnings
try:
    requests.packages.urllib3.disable_warnings()
except:
    pass

# Get the log handler
log = logging.getLogger('otx_misp')


class ImportException(Exception):
    pass

# MISP instance version
_misp_server_version = None


def misp_server_version(misp):
    """
    Retrieve the MISP instance version
    
    :param misp: MISP connection object
    :type misp: :class:`pymisp.PyMISP`
    :return: MISP instance version as string
    """
    global _misp_server_version
    if _misp_server_version is None:
        version = misp.get_version()
        _misp_server_version = version['version']
    return _misp_server_version


def tag_event(misp, event, tag):
    """
    Add a tag to a MISP event
    
    :param misp: MISP connection object
    :type misp: :class:`pymisp.PyMISP` 
    :param event: a MISP event
    :param tag: tag to add
    :return: None
    """
    if not hasattr(misp, '_otx_tags_cache'):
        misp._otx_tags_cache = misp.get_all_tags()['Tag']
    for exist_tag in misp._otx_tags_cache:
        if exist_tag['name'] == tag:
            tag_id = exist_tag['id']
            if 'EventTag' in event:
                for evt_tag in event['EventTag']:
                    if tag_id == evt_tag['id']:
                        log.info("\t - Tag already exists. Skipping:".format(tag))
                        return
            break
    if hasattr(misp, 'tag'):
        version = misp_server_version(misp).split('.')
        tag_version = '2.4.69'.split('.')
        for a, b in zip(version, tag_version):
            if a == b:
                continue
            elif int(a) > int(b):
                continue
            else:  # a < b
                misp.add_tag(event, tag)
                return
        misp.tag(event['Event']['uuid'], tag)
    else:
        misp.add_tag(event, tag)


def get_pulses(otx_api_key, from_timestamp=None):
    """
    Get the Pulses from Alienvault OTX

    :param otx_api_key: Alienvault OTX API key
    :type otx_api_key: string
    :param from_timestamp: only download Pulses after this date/time (None for all Pulses)
    :type from_timestamp: :class:`datetime.datetime` or ISO string or Unix timestamp
    :return: a list of Pulses (dict)
    """
    otx = OTXv2(otx_api_key)
    if from_timestamp is None:
        log.debug("Retrieving all Pulses (no timestamp)")
        pulses = otx.getall()
    elif isinstance(from_timestamp, int):
        dt = datetime.fromtimestamp(from_timestamp)
        from_timestamp = dt.isoformat()
        pulses = otx.getsince(from_timestamp)
    elif isinstance(from_timestamp, datetime):
        from_timestamp = from_timestamp.isoformat()
        pulses = otx.getsince(from_timestamp)
    elif isinstance(from_timestamp, basestring):
        pulses = otx.getsince(from_timestamp)
    else:
        raise ValueError("'from_timestamp' must be 'None', a datetime object or an ISO date string")
    return pulses


def get_pulses_iter(otx_api_key, from_timestamp=None):
    """
    Get the Pulses from Alienvault OTX and returns a generator

    :param otx_api_key: Alienvault OTX API key
    :type otx_api_key: string
    :param from_timestamp: only download Pulses after this date/time (None for all Pulses)
    :type from_timestamp: :class:`datetime.datetime` or ISO string or Unix timestamp
    :return: a generator of Pulses (dict)
    """
    otx = OTXv2(otx_api_key)
    if from_timestamp is None:
        log.debug("Retrieving all Pulses (no timestamp)")
        return otx.getall()
    elif isinstance(from_timestamp, int):
        dt = datetime.fromtimestamp(from_timestamp)
        from_timestamp = dt.isoformat()
    elif isinstance(from_timestamp, datetime):
        from_timestamp = from_timestamp.isoformat()
    elif not isinstance(from_timestamp, basestring):
        raise ValueError("'from_timestamp' must be 'None', a datetime object or an ISO date string")
    return otx.getsince_iter(from_timestamp)


def create_events(pulse_or_list, author=False, server=False, key=False, misp=False, distribution=0, threat_level=4,
                  analysis=2, publish=True, tlp=True, discover_tags=False, to_ids=False, author_tag=False,
                  bulk_tag=None, dedup_titles=False, stop_on_error=False):
    """
    Parse a Pulse or a list of Pulses and add it/them to MISP if server and key are present

    :param pulse_or_list: a Pulse or list of Pulses as returned by `get_pulses`
    :param author: Prepend the author to the Pulse name
    :type author: Boolean
    :param server: MISP server URL
    :param key: MISP API key
    :param misp: MISP connection object
    :type misp: :class:`pymisp.PyMISP`
    :param distribution: distribution of the MISP event (0-4)
    :param threat_level: threat level of the MISP object (1-4)
    :param analysis: analysis stae of the MISP object (0-2)
    :param publish: Is the MISP event should be published?
    :type publish: Boolean
    :param tlp: Add TLP level tag to event
    :type tlp: Boolean
    :param discover_tags: discover MISP tags from Pulse tags
    :type discover_tags: Boolean
    :param to_ids: Flag pulse attributes as being sent to an IDS
    :type to_ids: Boolean
    :param author_tag: Add the pulse author as an event tag
    :type author_tag: Boolean
    :param bulk_tag: A tag that will be added to all events for categorization (e.g. OTX)
    :type bulk_tag: String
    :param dedup_titles: Search MISP for an existing event title and update it, rather than create a new one
    :type dedup_titles: Boolean
    :return: a dict or a list of dict with the selected attributes
    """
    if not misp and (server and key):
        log.debug("Connection to MISP instance: {}".format(server))
        try:
            misp = pymisp.PyMISP(server, key, ssl=False, out_type='json')
        except pymisp.PyMISPError as ex:
            raise ImportException("Cannot connect to MISP instance: {}".format(ex.message))
        except Exception as ex:
            raise ImportException("Cannot connect to MISP instance, unknown exception: {}".format(ex.message))
    if discover_tags:
        def get_tag_name(complete):
            parts = complete.split('=')
            if not len(parts):
                return complete
            last = parts[-1]
            if not len(last):
                return complete
            if last[0] == '"':
                last = last[1:]
            if last[-1] == '"':
                last = last[:-1]
            return last.lower()
        raw_tags = misp.get_all_tags()
        tags = dict()
        for tag in raw_tags['Tag']:
            tags[get_tag_name(tag['name'])] = tag['name']
        misp.discovered_tags = tags

    if isinstance(pulse_or_list, (list, tuple)) or inspect.isgenerator(pulse_or_list):
        misp_events = []
        for pulse in pulse_or_list:
            try:
                misp_event = create_events(pulse, author=author, server=server, key=key, misp=misp,
                                           distribution=distribution, threat_level=threat_level, analysis=analysis,
                                           publish=publish, tlp=tlp, to_ids=to_ids, author_tag=author_tag,
                                           bulk_tag=bulk_tag, dedup_titles=dedup_titles, stop_on_error=stop_on_error)
                misp_events.append(misp_event)
            except Exception as ex:
                if stop_on_error:
                    raise
                name = ''
                if pulse and 'name' in pulse:
                    name = pulse['name']
                log.error("Cannot import pulse {}: {}".format(name, ex))
        return misp_events

    pulse = pulse_or_list
    if author:
        event_name = pulse['author_name'] + ' | ' + pulse['name']
    else:
        event_name = pulse['name']
    try:
        dt = date_parser.parse(pulse['created'])
    except (ValueError, OverflowError):
        log.error("Cannot parse Pulse 'created' date.")
        dt = datetime.utcnow()
    event_date = dt.strftime('%Y-%m-%d')
    log.info("## {name} - {date}".format(name=event_name, date=event_date))
    result_event = {
        'name': event_name,
        'date': event_date,
        'tags': list(),
        'attributes': {
            'hashes': {
                'md5': list(),
                'sha1': list(),
                'sha256': list(),
                'imphash': list(),
                'pehash': list()
            },
            'hostnames': list(),
            'domains': list(),
            'urls': list(),
            'ips': list(),
            'emails': list(),
            'mutexes': list(),
            'references': list(),
            'cves': list(),
            'filenames': list(),
            'yara': list()
        },
    }

    if misp:
        if not dedup_titles:
            event = misp.new_event(distribution, threat_level, analysis, event_name, date=event_date, published=publish)
        else:
            event = ''
            # Check if username is added to title
            # Build the title
            if author:
                event_name = pulse['author_name'] + ' | ' + pulse['name']
            else:
                event_name = pulse['name']

            # Search MISP for the title
            result = misp.search_index(eventinfo=event_name)
            if 'message' in result:
                if result['message'] == "No matches.":
                    event = misp.new_event(distribution, threat_level, analysis, event_name, date=event_date,
                                           published=publish)
            else:
                for evt in result['response']:
                    # If it exists, set 'event' to the event
                    if evt['info'] == event_name:
                        if 'SharingGroup' in evt:
                            del evt['SharingGroup']  # This deletes the SharingGroup from the list, thx SparkyNZL
                        event = {'Event': evt}
                        break
                if event == '':
                    # Event not found, even though search results were returned
                    # Build new event
                    event = misp.new_event(distribution, threat_level, analysis, event_name, date=event_date,
                                           published=publish)

        time.sleep(0.2)
        if tlp:
            tag = None
            if 'TLP' in pulse:
                tag = "tlp:{}".format(pulse['TLP'])
            elif 'tlp' in pulse:
                tag = "tlp:{}".format(pulse['tlp'])
            if tag is not None:
                log.info("\t - Adding tag: {}".format(tag))
                tag_event(misp, event, tag)
                result_event['tags'].append(tag)

        if author_tag:
            tag_event(misp, event, pulse['author_name'])

        if bulk_tag is not None:
            tag_event(misp, event, bulk_tag)

    if misp and hasattr(misp, 'discovered_tags') and 'tags' in pulse:
        for pulse_tag in pulse['tags']:
            if pulse_tag.lower() in misp.discovered_tags:
                tag = misp.discovered_tags[pulse_tag.lower()]
                log.info("\t - Adding tag: {}".format(tag))
                tag_event(misp, event, tag)
                result_event['tags'].append(tag)

    if 'references' in pulse:
        for reference in pulse['references']:
            if reference:
                log.info("\t - Adding external analysis link: {}".format(reference))
                if misp:
                    misp.add_named_attribute(event, 'link', reference, category='External analysis')
                result_event['attributes']['references'].append(reference)

    if misp and 'description' in pulse and isinstance(pulse['description'], six.text_type) and pulse['description']:
        log.info("\t - Adding external analysis comment")
        misp.add_named_attribute(event, 'comment', pulse['description'], category='External analysis')

    for ind in pulse['indicators']:
        ind_type = ind['type']
        ind_val = ind['indicator']
        ind_kwargs = {'to_ids': to_ids}

        if 'description' in ind and isinstance(ind['description'], six.text_type) and ind['description']:
            ind_kwargs['comment'] = ind['description']

        if ind_type == 'FileHash-SHA256':
            log.info("\t - Adding SHA256 hash: {}".format(ind_val))
            if misp:
                misp.add_hashes(event, sha256=ind_val, **ind_kwargs)
            result_event['attributes']['hashes']['sha256'].append(ind_val)

        elif ind_type == 'FileHash-SHA1':
            log.info("\t - Adding SHA1 hash: {}".format(ind_val))
            if misp:
                misp.add_hashes(event, sha1=ind_val, **ind_kwargs)
            result_event['attributes']['hashes']['sha1'].append(ind_val)

        elif ind_type == 'FileHash-MD5':
            log.info("\t - Adding MD5 hash: {}".format(ind_val))
            if misp:
                misp.add_hashes(event, md5=ind_val, **ind_kwargs)
            result_event['attributes']['hashes']['md5'].append(ind_val)

        elif ind_type == 'URI' or ind_type == 'URL':
            log.info("\t - Adding URL: {}".format(ind_val))
            if misp:
                misp.add_url(event, ind_val, **ind_kwargs)
            result_event['attributes']['urls'].append(ind_val)

        elif ind_type == 'domain':
            log.info("\t - Adding domain: {}".format(ind_val))
            if misp:
                misp.add_domain(event, ind_val, **ind_kwargs)
            result_event['attributes']['domains'].append(ind_val)

        elif ind_type == 'hostname':
            log.info("\t - Adding hostname: {}".format(ind_val))
            if misp:
                misp.add_hostname(event, ind_val, **ind_kwargs)
            result_event['attributes']['hostnames'].append(ind_val)

        elif ind_type == 'IPv4' or ind_type == 'IPv6':
            log.info("\t - Adding ip: {}".format(ind_val))
            if misp:
                misp.add_ipdst(event, ind_val, **ind_kwargs)
            result_event['attributes']['ips'].append(ind_val)

        elif ind_type == 'email':
            log.info("\t - Adding email: {}".format(ind_val))
            if misp:
                misp.add_email_dst(event, ind_val, **ind_kwargs)
            result_event['attributes']['emails'].append(ind_val)

        elif ind_type == 'Mutex':
            log.info("\t - Adding mutex: {}".format(ind_val))
            if misp:
                misp.add_mutex(event, ind_val, **ind_kwargs)
            result_event['attributes']['mutexes'].append(ind_val)

        elif ind_type == 'CVE':
            log.info("\t - Adding CVE: {}".format(ind_val))
            if misp:
                misp.add_named_attribute(event, 'vulnerability', ind_val, category='External analysis', **ind_kwargs)
            result_event['attributes']['cves'].append(ind_val)

        elif ind_type == 'FileHash-IMPHASH':
            log.info("\t - Adding IMPHASH hash: {}".format(ind_val))
            if misp:
                misp.add_named_attribute(event, 'imphash', ind_val, category='Artifacts dropped', **ind_kwargs)
            result_event['attributes']['hashes']['imphash'].append(ind_val)

        elif ind_type == 'FileHash-PEHASH':
            log.info("\t - Adding PEHASH hash: {}".format(ind_val))
            if misp:
                misp.add_named_attribute(event, 'pehash', ind_val, category='Artifacts dropped', **ind_kwargs)
            result_event['attributes']['hashes']['pehash'].append(ind_val)

        elif ind_type == 'FilePath':
            log.info("\t - Adding filename: {}".format(ind_val))
            if misp:
                misp.add_filename(event, ind_val, category='Artifacts dropped', **ind_kwargs)
            result_event['attributes']['filenames'].append(ind_val)

        elif ind_type == 'YARA':
            ind_title = ind.get('title', ind_val)
            ind_desc = ind.get('description', '')
            if ind_title == '':
                ind_title = ind_val
                if not ind_desc == '':
                    ind_kwargs['comment'] = ind_desc
            else:
                ind_kwargs['comment'] = "{} {}".format(ind_title, ind_desc)
            ind_val = ind.get('content', None)
            if ind_val is None or ind_val == "":
                log.warning("YARA indicator is empty: %s" % ind_title)
                continue
            log.info("\t - Adding YARA rule: {}".format(ind_title))
            if misp:
                misp.add_yara(event, ind_val, category='Artifacts dropped', **ind_kwargs)
            result_event['attributes']['yara'].append({'title': ind_title, 'content': ind_val})

        else:
            log.warning("Unsupported indicator type: %s" % ind_type)

    if misp and publish:
        event['Event']['published'] = False
        misp.publish(event)
    return result_event



"""
Module that contains the command line app.

Why does this file exist, and why not put this in __main__?

  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:

  - When you run `python -motx_misp` python will execute
    ``__main__.py`` as a script. That means there won't be any
    ``otx_misp.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there's no ``otx_misp.__main__`` in ``sys.modules``.

  Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration
"""
import argparse
import logging
import os.path
import sys
from datetime import datetime

from dateutil import parser as date_parser

from otx_misp.configuration import Configuration
from otx_misp import get_pulses_iter as get_pulses, create_events
from .otx import InvalidAPIKey, BadRequest

log = logging.getLogger('otx_misp')
console_handler = logging.StreamHandler()
log.addHandler(console_handler)


def timestamp(argument):
    try:
        unix_ts = int(argument)
        date = datetime.utcfromtimestamp(unix_ts)
    except ValueError:
        date = date_parser.parse(argument)
    return date


misp_distributions = ['organisation', 'community', 'connected', 'all']
misp_threat_levels = ['high', 'medium', 'low', 'undefined']
misp_analysis = ['initial', 'ongoing', 'completed']


def get_misp_type(choices, bias=0):
    def misp_type(argument):
        try:
            dist = int(argument)
        except ValueError:
            argument = argument.lower()
            dist = choices.index(argument)
        if 0 <= dist < len(choices):
            return dist + bias
        raise ValueError

    return misp_type


parser = argparse.ArgumentParser(description='Downloads OTX pulses and add them to MISP.')
parser.add_argument('-o', '--otx', help="Alienvault OTX API key", dest='otx')
parser.add_argument('-s', '--server', help="MISP server URL")
parser.add_argument('-m', '--misp', help='MISP API key', dest='misp')
parser.add_argument('-t', '--timestamp', help='Last import as Date/Time ISO format or UNIX timestamp', type=timestamp,
                    dest='timestamp', default=None)
parser.add_argument('-c', '--config-file', dest='config')
parser.add_argument('-w', '--write-config', help='Write the configuration file', action='store_true')
parser.add_argument('-a', '--author', help='Add the Pulse author name in the MISP Info field', action='store_true')
parser.add_argument('-u', '--update-timestamp', help='Updates the timestamp in the configuaration file',
                    action='store_const', const=True)
parser.add_argument('-n', '--no-publish', help="Don't publish the MISP event" , action='store_false', dest='publish')
parser.add_argument('-d', '--dry-run', help="Fetch the pulses but don't create MISP events. Use -v[v] to see details.",
                    action='store_true', dest='simulate')
parser.add_argument("-v", "--verbose", dest="verbose",
                    action="count", default=0,
                    help="Verbosity, repeat to increase the verbosity level.")
parser.add_argument('--no-tlp', help='No Traffic Light Protocol tag',
                    action='store_false', dest='tlp')
parser.add_argument('--discover-tags', help='Discover tags to add to MISP events',
                    action='store_true')
parser.add_argument('--to-ids', help='Mark IOCs as exportable to IDS',
                    action='store_true')
parser.add_argument('--distribution',
                    help="MISP distribution of events ({}), default: {}".format(','.join(misp_distributions),
                                                                                misp_distributions[0]),
                    type=get_misp_type(misp_distributions), default=None)
parser.add_argument('--threat-level',
                    help="MISP threat level of events ({}), default: {}".format(','.join(misp_threat_levels),
                                                                                misp_threat_levels[3]),
                    type=get_misp_type(misp_threat_levels, bias=1), default=None)
parser.add_argument('--analysis',
                    help="MISP analysis state of events ({}), default: {}".format(','.join(misp_analysis),
                                                                                  misp_analysis[2]),
                    type=get_misp_type(misp_analysis), default=None)
parser.add_argument('--author-tag', help="Add the pulse author as an event tag",
                    action='store_true')
parser.add_argument('--bulk-tag', help="Add a custom tag that will be added to all events (e.g. OTX)",
                    type=str)
parser.add_argument('--dedup-titles',
                    help="Search MISP for an existing event title and update it, rather than create a new one",
                    action='store_true')
parser.add_argument('--stop-on-error',
                    help="Stop import when an exception is raised",
                    action='store_true')


def main(args=None):
    args = parser.parse_args(args=args)
    if args.verbose == 1:
        log.setLevel('WARNING')
    elif args.verbose == 2:
        log.setLevel('INFO')
    elif args.verbose >= 3:
        log.setLevel('DEBUG')
    else:
        log.setLevel('ERROR')
    if args.simulate:
        if (not args.config or not os.path.isfile(args.config)) and not args.otx:
            log.error("You must either give an existing config file or your OTX API key with '--dry-run'.")
            sys.exit(4)
    elif (not args.config or not os.path.isfile(args.config)) and not (args.otx and args.server and args.misp):
        log.error("You must either give an existing config file or your API keys and the MISP server URL.")
        sys.exit(2)
    try:
        config = Configuration(args)
    except Exception as ex:
        log.error(ex.message)
        sys.exit(5)

    try:
        pulses = get_pulses(config.otx, from_timestamp=config.timestamp.isoformat())
    except InvalidAPIKey:
        log.error("Wrong API key: '{}'".format(config.otx))
        sys.exit(11)
    except ValueError as ex:
        log.error("Cannot use last import timestamp '{}'".format(config.timestamp.isoformat()))
        sys.exit(12)
    except BadRequest:
        log.error("Bad request")
        sys.exit(13)
    kwargs = {}
    if not config.simulate:
        kwargs = {
            'server': config.server,
            'key': config.misp,
            'distribution': config.distribution,
            'threat_level': config.threat_level,
            'analysis': config.analysis,
            'tlp': config.tlp,
            'discover_tags': config.discover_tags,
            'to_ids': config.to_ids,
            'author_tag': config.author_tag,
            'bulk_tag': config.bulk_tag,
            'dedup_titles': config.dedup_titles,
            'stop_on_error': config.stop_on_error
        }
        try:
            import pymisp
        except ImportError:
            log.error('PyMISP is not installed. Aborting.')
            sys.exit(20)
    try:
        create_events(pulses, author=config.author, **kwargs)
    except Exception as ex:
        log.error("Error: {}".format(ex))
        sys.exit(21)
    if config.write_config or config.update_timestamp:
        if args.config:
            with open(args.config, 'w') as f:
                config.write(f)
        else:
            config.write(sys.stdout)



#!/usr/bin/env python

from distutils.core import setup

setup(name='OTXv2',
      version='1.1',
      description='AlienVault OTX API',
      author='AlienVault Team',
      author_email='otx@alienvault.com',
      url='https://github.com/AlienVault-Labs/OTX-Python-SDK',
      py_modules=['OTXv2','IndicatorTypes'],
      install_requires=['simplejson']
      )



"""
All files in this directory are Copyright 2015 AlienVault, Inc.

This files are downloaded from the OTX Python SDK repository:
https://github.com/AlienVault-Labs/OTX-Python-SDK
"""

from .OTXv2 import OTXv2, InvalidAPIKey, BadRequest

__all__ = ['OTXv2', 'InvalidAPIKey', 'BadRequest']


from __future__ import unicode_literals

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import logging
import os.path
from datetime import datetime

import io
from dateutil import parser as date_parser


class ConfigurationError(Exception):
    pass


class Configuration(object):
    """ Object holding the configuration """
    properties = {'otx': True, 'misp': True, 'server': True, 'timestamp': False, 'author': False,
                  'distribution': False, 'threat_level': False, 'analysis': False, 'update_timestamp': False,
                  'publish': False, 'tlp': False, 'discover_tags': False, 'to_ids': False, 'author_tag': False,
                  'bulk_tag': False, 'dedup_titles': False, 'stop_on_error': False}
    simulation_properties = {'otx': True, 'misp': True, 'server': True, 'timestamp': False, 'author': False,
                             'distribution': False, 'threat_level': False, 'analysis': False, 'update_timestamp': False,
                             'publish': False, 'tlp': False, 'discover_tags': False, 'to_ids': False,
                             'author_tag': False, 'bulk_tag': False, 'dedup_titles': False, 'stop_on_error': False}
    defaults = {'distribution': 0, 'threat_level': 4, 'analysis': 2, 'timestamp': datetime.utcfromtimestamp(0),
                'update_timestamp': False}
    config_section = 'otx_misp'

    def __init__(self, arguments):
        self.config = configparser.SafeConfigParser(allow_no_value=True)
        if arguments.config and os.path.isfile(arguments.config):
            self.config.read(arguments.config)
        if not self.config.has_section(self.config_section):
            self.config.add_section(self.config_section)
        self.arguments = arguments
        self.original_config = self._clone_config(self.config)
        self._populate_config()

    @staticmethod
    def _clone_config(config):
        clone = configparser.SafeConfigParser(allow_no_value=True)
        for section in config.sections():
            clone.add_section(section)
            for option in config.options(section):
                clone.set(section, option, config.get(section, option))
        return clone

    def _populate_config(self):
        if self.arguments.simulate:
            self.properties = self.simulation_properties
        for key, required in list(self.simulation_properties.items()):
            value = getattr(self.arguments, key, None)
            if isinstance(value, bool):
                if value:
                    value = 'yes'
                else:
                    value = 'no'
            elif isinstance(value, (list, tuple)):
                value = ','.join(value)
            elif isinstance(value, datetime):
                value = value.isoformat()
            elif isinstance(value, int):
                value = '{}'.format(value)
            if value is not None:
                self.config.set(self.config_section, key, value)
            if required:
                try:
                    value = self.config.get(self.config_section, key)
                except configparser.NoOptionError:
                    raise ConfigurationError("Missing required parameter: '--{}'".format(key))
                if value is None:
                    raise ConfigurationError("Missing required parameter: '--{}'".format(key))

    def __getattr__(self, item):
        if item not in self.arguments:
            raise AttributeError
        value = getattr(self.arguments, item, None)
        if value is None and item in self.defaults:
            value = self.defaults[item]
        if item not in self.properties or not self.config.has_option(self.config_section, item):
            return value
        if isinstance(value, bool):
            return self.config.getboolean(self.config_section, item)
        elif isinstance(value, (list, tuple)):
            parameter = self.config.get(self.config_section, item)
            return parameter.split(',')
        elif isinstance(value, datetime):
            parameter = self.config.get(self.config_section, item)
            if parameter is not None:
                try:
                    return date_parser.parse(parameter)
                except:
                    pass
        elif isinstance(value, int):
            return self.config.getint(self.config_section, item)
        return self.config.get(self.config_section, item)

    def write(self, fp):
        if self.write_config:
            config = self.config
        else:
            config = self.original_config
        if self.update_timestamp:
            config.set(self.config_section, 'timestamp', datetime.utcnow().isoformat())
        return config.write(fp)



class IndicatorTypes(object):
    def __init__(self, name, description):
        self.name = name
        self.description = description


IPv4 = IndicatorTypes(name="IPv4",
                      description="An IPv4 address indicating the online location of a server or other computer.")
IPv6 = IndicatorTypes(name="IPv6",
                      description="An IPv6 address indicating the online location of a server or other computer.")
DOMAIN = IndicatorTypes(name="domain",
                        description="A domain name for a website or server. Domains encompass a series of hostnames.")
HOSTNAME = IndicatorTypes(name="hostname", description="The hostname for a server located within a domain.")
EMAIL = IndicatorTypes(name="email", description="An email associated with suspicious activity.")
URL = IndicatorTypes(name="URL",
                     description=" Uniform Resource Location (URL) summarizing"
                                 " the online location of a file or resource.")
URI = IndicatorTypes(name="URI",
                     description="Uniform Resource Indicator (URI) describing"
                                 " the explicit path to a file hosted online.")
FILE_HASH_MD5 = IndicatorTypes(name="FileHash-MD5",
                               description="A MD5-format hash that summarizes"
                                           " the architecture and content of a file.")
FILE_HASH_SHA1 = IndicatorTypes(name="FileHash-SHA1",
                                description="A SHA-format hash that summarizes"
                                            " the architecture and content of a file.")
FILE_HASH_SHA256 = IndicatorTypes(name="FileHash-SHA256",
                                  description="A SHA-256-format hash that summarizes"
                                              " the architecture and content of a file.")
FILE_HASH_PEHASH = IndicatorTypes(name="FileHash-PEHASH",
                                  description="A PEPHASH-format hash that summarizes the"
                                              " architecture and content of a file.")
FILE_HASH_IMPHASH = IndicatorTypes(name="FileHash-IMPHASH",
                                   description="An IMPHASH-format hash that summarizes"
                                               " the architecture and content of a file.")
CIDR = IndicatorTypes(name="CIDR",
                      description="Classless Inter-Domain Routing (CIDR) address, which"
                                  " describes both a server's IP address and the network"
                                  " architecture (routing path) surrounding that server.")
FILE_PATH = IndicatorTypes(name="FilePath", description="A unique location in a file system.")
MUTEX = IndicatorTypes(name="Mutex",
                       description="The name of a mutex resource describing the"
                                   " execution architecture of a file.")
CVE = IndicatorTypes(name="CVE",
                     description="Common Vulnerability and Exposure (CVE) entry"
                                 " describing a software vulnerability that can be"
                                 " exploited to engage in malicious activity.")
all_types = [IPv4,
             IPv6,
             DOMAIN,
             HOSTNAME,
             EMAIL,
             URL,
             URI,
             FILE_HASH_MD5,
             FILE_HASH_SHA1,
             FILE_HASH_SHA256,
             FILE_HASH_PEHASH,
             FILE_HASH_IMPHASH,
             CIDR,
             FILE_PATH,
             MUTEX,
             CVE]


def to_name_list(indicator_type_list):
    return [indicator_type.name for indicator_type in indicator_type_list]
